Package: GraphExperiment
Title: S4 Class for Quantitative Data and Associated Networks
Version: 1.0.1
Date: 2026-03-01
Authors@R: 
    c(
    person(given = "Fabricio",
           family = "Almeida-Silva",
           role = c("aut", "cre"),
           email = "fabricio_almeidasilva@hotmail.com",
           comment = c(ORCID = "0000-0002-5314-2964")),
    person(given = "Yves",
           family = "Van de Peer",
           role = "aut",
           email = "yves.vandepeer@psb.vib-ugent.be",
           comment = c(ORCID = "0000-0003-4327-3730"))
    )
Description: GraphExperiment provides users and developers with an S4 class
    that extends `SingleCellExperiment` by offering infrastructure to
    store and retrieve networks (`igraph` objects) representing how assay 
    features and/or observations are associated with each other. The class was 
    designed to store networks inferred from high-dimensional quantitative 
    data, with feature-feature networks including gene coexpression 
    networks (GCNs), gene regulatory networks (GRNs), and
    co-abundance networks (from proteomics and metabolomics), and
    observation-observation network including cell-cell distances,
    species-species relationships, and sample-sample similarities.
License: GPL-3
URL: https://github.com/almeidasilvaf/GraphExperiment
BugReports: https://support.bioconductor.org/tag/GraphExperiment
Encoding: UTF-8
biocViews: DataRepresentation, DataImport, Infrastructure,
        GeneExpression, Transcriptomics, Network, SingleCell
Depends: SingleCellExperiment, igraph
Imports: methods, SummarizedExperiment, BiocBaseUtils, S4Vectors
Suggests: knitr, BiocStyle, testthat, rmarkdown, covr, sessioninfo
Config/testthat/edition: 3
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/GraphExperiment
git_branch: RELEASE_3_23
git_last_commit: 4494c10
git_last_commit_date: 2026-05-22
Repository: Bioconductor 3.23
Date/Publication: 2026-05-24
NeedsCompilation: no
Packaged: 2026-05-24 22:15:12 UTC; biocbuild
Author: Fabricio Almeida-Silva [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5314-2964>),
  Yves Van de Peer [aut] (ORCID: <https://orcid.org/0000-0003-4327-3730>)
Maintainer: Fabricio Almeida-Silva <fabricio_almeidasilva@hotmail.com>
